@@ -19,10 +19,6 @@
 
 /* configuration header file for compiling under Microsoft Windows */
 
-/* update package version here */
-#define PACKAGE "jbig2dec"
-#define VERSION "0.16"
-
 #ifdef _MSC_VER
 
 /* VS 2012 and later have stdint.h */
@@ -45,6 +41,15 @@ typedef unsigned __int64 uint64_t;
 /* VS 2008 and later have vsnprintf */
 # if _MSC_VER < 1500
 #  define vsnprintf _vsnprintf
+/* Previously we defined inline as nothing for 2005 and below */
+#  define inline
+#else
+/* VS 2008 has __inline but not inline, later versiosn (unknown exactly where) define inline
+ * so cater for it being defined already.
+ */
+# if !(defined(inline))
+#  define inline __inline
+# endif
 # endif
 
 /* VS 2014 and later have (finally) snprintf */
