@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 /*
@@ -48,28 +48,30 @@ jbig2_arith_iaid_ctx_new(Jbig2Ctx *ctx, int SBSYMCODELEN)
     int ctx_size = 1 << SBSYMCODELEN;
 
     if (result == NULL) {
-        jbig2_error(ctx, JBIG2_SEVERITY_FATAL, -1, "failed to allocate storage in jbig2_arith_iaid_ctx_new");
-        return result;
+        jbig2_error(ctx, JBIG2_SEVERITY_FATAL, -1, "failed to allocate IAID arithmetic coding state");
+        return NULL;
     }
 
     result->SBSYMCODELEN = SBSYMCODELEN;
     result->IAIDx = jbig2_new(ctx, Jbig2ArithCx, ctx_size);
-    if (result->IAIDx != NULL) {
-        memset(result->IAIDx, 0, ctx_size);
-    } else {
-        jbig2_error(ctx, JBIG2_SEVERITY_FATAL, -1, "failed to allocate symbol ID storage in jbig2_arith_iaid_ctx_new");
+    if (result->IAIDx == NULL)
+    {
+        jbig2_free(ctx->allocator, result);
+        jbig2_error(ctx, JBIG2_SEVERITY_FATAL, -1, "failed to allocate symbol ID in IAID arithmetic coding state");
+        return NULL;
     }
 
+    memset(result->IAIDx, 0, ctx_size);
     return result;
 }
 
 /* A.3 */
 /* Return value: -1 on error, 0 on normal value */
 int
-jbig2_arith_iaid_decode(Jbig2ArithIaidCtx *ctx, Jbig2ArithState *as, int32_t *p_result)
+jbig2_arith_iaid_decode(Jbig2Ctx *ctx, Jbig2ArithIaidCtx *actx, Jbig2ArithState *as, int32_t *p_result)
 {
-    Jbig2ArithCx *IAIDx = ctx->IAIDx;
-    int SBSYMCODELEN = ctx->SBSYMCODELEN;
+    Jbig2ArithCx *IAIDx = actx->IAIDx;
+    int SBSYMCODELEN = actx->SBSYMCODELEN;
     int PREV = 1;
     int D;
     int i;
@@ -78,7 +80,7 @@ jbig2_arith_iaid_decode(Jbig2ArithIaidCtx *ctx, Jbig2ArithState *as, int32_t *p_
     for (i = 0; i < SBSYMCODELEN; i++) {
         D = jbig2_arith_decode(as, &IAIDx[PREV]);
         if (D < 0)
-            return -1;
+            return jbig2_error(ctx, JBIG2_SEVERITY_FATAL, -1, "failed to decode IAIDx code");
 #ifdef VERBOSE
         fprintf(stderr, "IAID%x: D = %d\n", PREV, D);
 #endif
