@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 /*
@@ -33,12 +33,14 @@
 
 #include "jbig2.h"
 #include "jbig2_image.h"
+#include "jbig2_image_rw.h"
 
-int main(int argc, char *argv[])
+int
+main(int argc, char *argv[])
 {
     Jbig2Ctx *ctx;
     Jbig2Image *image;
-    int error;
+    int code;
 
     /* we need a context for the allocators */
     ctx = jbig2_ctx_new(NULL, 0, NULL, NULL, NULL);
@@ -49,17 +51,17 @@ int main(int argc, char *argv[])
     }
 
     image = jbig2_image_read_pbm_file(ctx, argv[1]);
-    if(image == NULL) {
+    if (image == NULL) {
         fprintf(stderr, "error reading pbm file '%s'\n", argv[1]);
         return 1;
     } else {
         fprintf(stderr, "converting %dx%d image to png format\n", image->width, image->height);
     }
 
-    error = jbig2_image_write_png_file(image, argv[2]);
-    if (error) {
-        fprintf(stderr, "error writing png file '%s' error %d\n", argv[2], error);
+    code = jbig2_image_write_png_file(image, argv[2]);
+    if (code) {
+        fprintf(stderr, "error writing png file '%s' error %d\n", argv[2], code);
     }
 
-    return (error);
+    return (code);
 }
